#needlessly overwrought output interface
using LabelledArrays
using OnlineStats
using Plots
import OnlineStats.fit!

"""
AbstractRecorder is the type that all "Recorder" types must extend for `fit!` (and possibly other stuff) to dispatch correctly.

The point of using different recorder types is that we only want to save the output that we need, and that output will vary depending on the specific experiment.

AbstractRecorders can be parameterized to have any element type, so when they are parameterized with T<:OnlineStat, we can use `fit!` to collect statistics about a bunch of Recorders into a single Recorder.
"""
abstract type AbstractRecorder{ElType} end

"""
DebugRecorder should store everything we might want to know about the model output. 
"""
struct DebugRecorder{ElType,ArrT1<:AbstractArray{ElType},ArrT2<:AbstractArray{ElType},ArrT3<:AbstractArray{ElType}} <: AbstractRecorder{ElType}
    recorded_status_totals::ArrT1
    daily_cases_by_age::ArrT3
    total_vaccinators::ArrT2
    mean_time_since_last_notification::ArrT2
    daily_immunized_by_age::ArrT3
end
# struct AgeSpecificVaccination{ElType,ArrType1<:AbstractArray{ElType}} <: AbstractRecorder{ElType}
#     new_ymo_vaccinations::ArrType1
#     daily_cases::ArrType2
#     total_vaccinators::ArrType2
# end
"""
Initialize an empty DebugRecorder. We use a labelledarray for the state vector, so the individal timeseries can be accessed by name.
"""
function DebugRecorder(sim_length)
    state_totals = @LArray Array{Int}(undef,4,sim_length) (S = (1,:),I = (2,:),R = (3,:), V = (4,:))
    total_vaccinators = Vector{Int}(undef,sim_length)
    mean_time_since_last_notification = Vector{Int}(undef,sim_length)
    daily_cases= Vector{Int}(undef,sim_length)
    daily_ymo_by_age = @LArray Array{Int}(undef,3,sim_length) (Y = (1,:),M = (2,:),O = (3,:))
    
    daily_cases_by_age = @LArray Array{Int}(undef,3,sim_length) (Y = (1,:),M = (2,:),O = (3,:))
    daily_immunized_by_age = @LArray Array{Int}(undef,3,sim_length) (Y = (1,:),M = (2,:),O = (3,:))
    return DebugRecorder(
        state_totals,
        daily_cases_by_age,
        total_vaccinators,
        mean_time_since_last_notification,
        daily_immunized_by_age
    )

end

"""
Initialize a DebugRecorder filled with (copies) of val. I should find a nicer way to combine both constructors.
"""
function DebugRecorder(val::T, sim_length) where T
    totals = [copy(val) for i in 1:4, j in 1:sim_length]
    state_totals = @LArray totals (S = (1,:),I = (2,:),R = (3,:), V = (4,:))
    total_vaccinators = [copy(val) for j in 1:sim_length]
    mean_time_since_last_notification = [copy(val) for j in 1:sim_length]
    ymo = [copy(val) for i in 1:3, j in 1:sim_length]

    totals_ymo = [copy(val) for i in 1:3, j in 1:sim_length]
    daily_cases_by_age = @LArray deepcopy(totals_ymo) (Y = (1,:),M = (2,:),O = (3,:))
    daily_immunized_by_age = @LArray deepcopy(totals_ymo) (Y = (1,:),M = (2,:),O = (3,:))
    return DebugRecorder(
        state_totals,
        daily_cases_by_age,
        total_vaccinators,
        mean_time_since_last_notification,
        daily_immunized_by_age
    )
end

function record!(t,modelsol, recorder::DebugRecorder)
    recorder.total_vaccinators[t] =  modelsol.daily_vaccinators
    recorder.daily_cases_by_age[:,t] .= modelsol.daily_cases_by_age
    recorder.recorded_status_totals[:,t] .= modelsol.status_totals
    alerts = filter(>(0),modelsol.time_of_last_alert)
    if !isempty(alerts)
        mean_alert_time = mean(t .- alerts) 
        recorder.mean_time_since_last_notification[t] = round(Int,mean_alert_time) 
    else
        recorder.mean_time_since_last_notification[t] = 0
    end
    # display(modelsol.daily_immunizations_by_age)
    recorder.daily_immunized_by_age[:,t] .= modelsol.daily_immunized_by_age
end

function record!(t,modelsol, recorder::Nothing)
    #do nothing
end


function mean_solve(samples,parameter_tuple,recorder)
    stat_recorder = recorder(Variance(), parameter_tuple.sim_length)
    output_recorder = recorder(parameter_tuple.sim_length)
    for i in 1:samples
        abm(parameter_tuple,output_recorder)
        fit!(stat_recorder,output_recorder)
    end
    return stat_recorder
end

function OnlineStats.fit!(stat_type::R,pt) where {T,OS<:OnlineStat{T}, R<:AbstractRecorder{OS}}
    for field in fieldnames(R)
        stat_field = getfield(stat_type,field)
        sample_field = getfield(pt,field)
        for (k,(stat_entry,sample_entry)) in enumerate(zip(stat_field,sample_field))
            fit!(stat_entry,sample_entry)
        end
    end
end

using Printf

const ts_colors = cgrad(:PuBu_9)

function plot_model(varname,univariate_series, output_list::Vector{T},infection_begin,vac_begin) where T<:DebugRecorder
    
    sim_length = length(output_list[1].recorded_status_totals.S)
    ts_list(data) = [
        (data.recorded_status_totals.S, "Susceptible over time"),
        (data.recorded_status_totals.R, "Recovered over time"),
        (data.total_vaccinators,  "No. vaccinators over time"),
        (data.mean_time_since_last_notification, "Mean time since last notification"),
        (data.daily_cases_by_age.Y,"Daily (incident) Y cases"),
        (data.daily_cases_by_age.M,"Daily (incident) M cases"),
        (data.daily_cases_by_age.O,"Daily (incident) O cases"),
        (data.daily_immunized_by_age.Y, "new Y vaccinations each day"),
        (data.daily_immunized_by_age.M, "new M vaccinations each day"),
        (data.daily_immunized_by_age.O, "new O vaccinations each day"),
    ]

    l = length(ts_list(output_list[1]))
    plts = [plot() for i=1:l]
    for (i,(p,data)) in enumerate(zip(univariate_series, output_list))
        # display(p[varname])
        if !isnothing(varname) 
            p_val = @sprintf "%.4f" (p[varname])
            labelname = "$varname = $(p_val)"
        else
            labelname = nothing
        end
        for (plt,(ts,title)) in zip(plts,ts_list(data))
            plot!(plt, mean.(ts); ribbon = std.(ts),
                    label = labelname, line_z = i, color=:blues,
                    ylabel = "no. of people",
                    colorbar = false,
                    title = title,
            ) 
        end
    end
    plot!(plts[begin]; legends = :topright)
    for p in plts
        vline!(p,[infection_begin]; label = "infection begin", line =:dot)
        vline!(p,[vac_begin]; label = "vaccination begin",line = :dot)
    end
    return plot(plts...;layout = (l,1),size=(800,400*l),leftmargin = 12Plots.mm)
end
